<?php

namespace Teknisa\Libs\Util;

use Doctrine\DBAL\DriverManager;
use Zeedhi\Framework\Session\Session;

class ConnectionFactory extends \Zeedhi\Framework\ORM\ConnectionFactory {
    
    protected static function shouldUseNewCrypt($connectionParams){
        if(isset($connectionParams["newCrypt"]) && $connectionParams["newCrypt"]){
            if(!isset($connectionParams["cryptKey"])) {
                throw new \Exception("The crypt key is required");
            }
            return true;
        } 
        return false;
    }
    
    public static function updateConnectionParams(array $connectionParams, $config = null, $eventManager = null, Session $session = null){
        $nrOrg = false;
        if($session) {
            $nrOrg = $session->has('nrorg') ? $session->get('nrorg') : $session->get('NRORG');

            //Set current DBNAME in session for use in login session
            if(isset($connectionParams['sessionMode'])) {
                $encryptedDBName = self::shouldUseNewCrypt($connectionParams) ? 
                        hash('sha3-512', $connectionParams['dbname']) :
                            Utilities::legacyEncrypt($connectionParams['dbname'],'zeedhi', "\0");
                $session->set('CURRENT_DBNAME', preg_replace('/[^A-Za-z0-9\-]/', '', $encryptedDBName));
            } else {
                $encryptedUser = self::shouldUseNewCrypt($connectionParams) ? 
                        hash('sha3-512', $connectionParams['user']) 
                        : Utilities::legacyEncrypt($connectionParams['user'],'zeedhi', "\0");
                $session->set('CURRENT_DBNAME', preg_replace('/[^A-Za-z0-9\-]/', '',$encryptedUser));
            }
        }

        if ($nrOrg && isset($connectionParams['useVpd']) && $connectionParams['useVpd'] == 'true') {
            if (isset($connectionParams['vpdWithWallet']) && $connectionParams['vpdWithWallet'] == 'true') {
                unset($connectionParams['host']);
                unset($connectionParams['port']);
                $connectionParams['user'] = '/';
                $connectionParams['password'] = '';
                $connectionParams['dbname'] = 'USR_ORG_' . $nrOrg;
                $connectionParams['sessionMode'] = -2147483648;
            } else {
                $connectionParams['user'] = 'USR_ORG_' . $nrOrg;
                $connectionParams['password'] = $connectionParams['vpdPassword'];
                if (isset($connectionParams['isWallet']) && $connectionParams['isWallet'] == 'true') {
                    $connectionParams['dbname'] = isset($connectionParams['vpdDBNAME'])
                        ? $connectionParams['vpdDBNAME']
                        : $connectionParams['vpdDBName'];
                    $connectionParams['dbname'] = $connectionParams['vpdDBNAME'];
                    $connectionParams['host'] = $connectionParams['vpdHost'];
                    $connectionParams['port'] = $connectionParams['vpdPort'];
                    $connectionParams['service'] = $connectionParams['vpdService'];
                    unset($connectionParams['sessionMode']);
                }
            }
        }

        if(empty($connectionParams["password"])) {
            $connectionParams["password"] = '';
        } else if(isset($connectionParams["isEncrypted"]) && $connectionParams["isEncrypted"] == 'true') {
            if(self::shouldUseNewCrypt($connectionParams)) {
                $connectionParams["password"] = self::decrypt($connectionParams["password"], $connectionParams["cryptKey"]);
            } else {
                $connectionParams["password"] = Utilities::legacyDecrypt($connectionParams["password"],'zeedhi',"\0");
            }
            $connectionParams["isEncrypted"] = false;
        }

        return DriverManager::getConnection($connectionParams, $config, $eventManager);
    }
}